const tableData = {
    "proksee-batch_version": "0.5.1",
    "run_date": "2024-03-11 20:12:32 UTC",
    "input_dir": "/path/to/example_data",
    "genomes": [
        {
            "code_name": "genome_1",
            "name": "Shigella_flexneri_2a_str._301",
            "accession": "NC_004337.2",
            "description": "Shigella flexneri 2a str. 301 chromosome, complete genome",
            "total_size": 4828820,
            "num_contigs": 2,
            "gc_content": 0.5065,
            "files": {
                "genbank": [
                    "GCF_000006925.2_ASM692v2_genomic.gbff"
                ],
                "json": [],
                "blast": [
                    "blast_against_s_flexneri.txt"
                ],
                "bed": [
                    "s_flexneri_features.bed"
                ],
                "vcf": [],
                "gff": [
                    "GCF_000006925.2_ASM692v2_genomic.gff"
                ]
            }
        },
        {
            "code_name": "genome_2",
            "name": "Escherichia_coli_str._K-12_substr._MG1655",
            "accession": "NC_000913.3",
            "description": "Escherichia coli str. K-12 substr. MG1655, complete genome",
            "total_size": 4641652,
            "num_contigs": 1,
            "gc_content": 0.5079,
            "files": {
                "genbank": [
                    "GCF_000005845.2_ASM584v2_genomic.gbff"
                ],
                "json": [],
                "blast": [
                    "blast_against_e_coli.txt"
                ],
                "bed": [
                    "e_coli_features.bed"
                ],
                "vcf": [
                    "e_coli_variants.vcf"
                ],
                "gff": [
                    "GCF_000005845.2_ASM584v2_genomic.gff"
                ]
            }
        },
        {
            "code_name": "genome_3",
            "name": "Klebsiella_aerogenes",
            "accession": "NZ_CP041925.1",
            "description": "Klebsiella aerogenes strain Ka37751 chromosome, complete genome",
            "total_size": 5249267,
            "num_contigs": 2,
            "gc_content": 0.5491,
            "files": {
                "genbank": [
                    "GCF_007632255.1_ASM763225v1_genomic.gbff"
                ],
                "json": [],
                "blast": [
                    "blast_against_k_aerogenes.txt"
                ],
                "bed": [
                    "k_aerogenes_features.bed"
                ],
                "vcf": [],
                "gff": [
                    "GCF_007632255.1_ASM763225v1_genomic.gff"
                ]
            }
        },
        {
            "code_name": "genome_4",
            "name": "Salmonella_enterica_subsp._enterica_serovar_Typhimurium_str._LT2",
            "accession": "NC_003197.2",
            "description": "Salmonella enterica subsp. enterica serovar Typhimurium str. LT2, complete genome",
            "total_size": 4951383,
            "num_contigs": 2,
            "gc_content": 0.5224,
            "files": {
                "genbank": [
                    "GCF_000006945.2_ASM694v2_genomic.gbff"
                ],
                "json": [],
                "blast": [
                    "blast_against_s_enterica.txt"
                ],
                "bed": [
                    "s_enterica_features.bed"
                ],
                "vcf": [],
                "gff": [
                    "GCF_000006945.2_ASM694v2_genomic.gff"
                ]
            }
        }
    ]
};